import 'package:flutter/material.dart';

class FrameImage extends StatefulWidget {
  final List<String> images;
  final int interval;
  final double width;
  final double height;
  final BoxFit fit;
  final bool autoStart;

  FrameImage({
    Key key,
    @required this.images,
    this.interval = 50,
    this.width,
    this.height,
    this.fit,
    this.autoStart = true,
  }): super(key: key);

  @override
  FrameImageState createState() => FrameImageState();
}

class FrameImageState extends State<FrameImage> with TickerProviderStateMixin {
  AnimationController _animationController;
  Animation<double> _animation;

  List<Image> _imageWidgets;
  List<String> _images;
  int _currentFrame = 0;
  bool _isRunning = false;

  bool get isRunning => _isRunning;

  @override
  void initState() {
    super.initState();
    _initData(widget.images);
    _initAnimation();
  }

  @override
  void dispose() {
    _isRunning = false;
    _releaseAnimation();
    super.dispose();
  }

  void _initData(List<String> images) {
    this._images = images ?? [];
    this._currentFrame = 0;
    this._imageWidgets = new List(_images.length);
    this._isRunning = widget.autoStart;
  }

  void _initAnimation() {
    if (_images.isEmpty) {
      _releaseAnimation();
      return ;
    }
    _animationController = new AnimationController(
      duration: Duration(milliseconds: widget.interval * _imageWidgets.length),
      vsync: this,
    );
    _animation = new Tween<double>(begin: 0, end: _imageWidgets.length.toDouble() - 1).animate(_animationController);
    _animation..addListener(() {
      int index = _animation.value.floor();
      if (index != _currentFrame) {
        _currentFrame = index;
        setState(() { });
      }
    })..addStatusListener((status) {
      if (status == AnimationStatus.completed) {
        _animationController.repeat();
      }
    });
    if (_isRunning) {
      startAnimation();
    }
  }

  void _releaseAnimation() {
    _animationController?.dispose();
    _animationController = null;
    _animation = null;
  }

  void startAnimation() {
    _isRunning = true;
    _animationController?.forward();
  }

  void stopAnimation() {
    _isRunning = false;
    _animationController?.stop();
  }

  void reStartAnimation() {
    _isRunning = true;
    _animationController?.reset();
    _animationController?.forward();
  }

  void setImages(List<String> images) {
    if (_images != images) {
      _animationController?.dispose();
      _initData(images);
      _initAnimation();
      setState(() {});
    }
  }

  Widget _getImageWidget(int index) {
    Image image = _imageWidgets[index];
    if(image == null) {
      image = Image.asset(
        _images[index],
        width: widget.width,
        height: widget.height,
        fit: widget.fit ?? BoxFit.contain,
      );
      _imageWidgets[index] = image;
    }
    return image;
  }

  Widget _preLoadImage(int index) {
    if (index < 0 || index > _images.length - 1) return null;
    if (_imageWidgets[index] != null) return null;
    return Image.asset(
      _images[index],
      width: 0,
      height: 0
    );
  }

  @override
  void didUpdateWidget(FrameImage oldWidget) {
    super.didUpdateWidget(oldWidget);
    // 如果配置发生了变化，则清空缓存
    if(_images != widget.images
        || oldWidget.width != widget.width
        || oldWidget.height != widget.height
    ) {
      print("didUpdateWidget: FrameImage 配置发生了改变");
      _initData(widget.images);
      _initAnimation();
    }
  }

  @override
  Widget build(BuildContext context) {
//    print("build: FrameImage _currentFrame: $_currentFrame");

    List<Widget> imageWidgets = [];
    if (_images.isNotEmpty) {
      // 预加载两张图片，防止闪屏
      Widget widget1 = _preLoadImage(_currentFrame + 2);
      Widget widget2 = _preLoadImage(_currentFrame + 1);
      if (widget1 != null) {
        imageWidgets.add(widget1);
      }
      if (widget2 != null) {
        imageWidgets.add(widget2);
      }
      imageWidgets.add(_getImageWidget(_currentFrame));
    }
    return Stack(
        alignment: AlignmentDirectional.center,
        children: imageWidgets
    );

//    return _getImageWidget(_currentFrame);
  }
}
